<?php

class paybylink_Payment extends WC_Payment_Gateway
{
    private $paymentUrl;
        
    public function setControl($control) {
        $this -> control =  $control;
    }

    public function setEmail($email) {
        $this -> email = $email;
    }

    public function setNotifyUrl($url) {
        $this -> notifyURL = $url;
    }

    public function setReturnUrlSuccess($url) {
        $this -> returnUrlSuccess = $url;
    }

     public function __construct( )
    {
        $this -> id                 = 'paybylink_payment';
        $this -> icon               = plugins_url( 'img/logo.png', __FILE__ );
        $this -> has_fields         = false;
        $this -> method_title       = '[Paybylink.pl] Płatność online oraz BLIK';
        $this -> method_description = 'Pośrednik finansowy udostępniający największą ilość banków w Polsce. Dodatkowo bramka umożliwia wpłaty PayPal oraz BLIK.';
        $this -> init_form_fields();
        $this -> init_settings();
        $this -> title = $this -> get_option( 'title' );
        add_action( 'woocommerce_api_' . $this -> id, array(
             $this,
            'paybylink_callback' 
        ) );
        add_action( 'woocommerce_update_options_payment_gateways_' . $this -> id, array(
             $this,
            'process_admin_options' 
        ) );
    }


    public function paybylink_callback()
    {
        $response = file_get_contents('php://input');
        $response = json_decode($response, true);


        if(!isset($response['transactionId']) OR empty($response['transactionId'])) {
            echo 'not found transactionId';
            $error = true;
        }   

        if(!isset($response['control']) OR empty($response['control'])) {
            echo 'not found control';
            $error = true;
        }   

        if(!isset($response['email']) OR empty($response['email'])) {
            echo 'not found email';
            $error = true;
        }   

        if(!isset($response['amountPaid']) OR empty($response['amountPaid'])) {
            echo 'not found amountPaid';
            $error = true;
        }   

        if(!isset($response['notificationAttempt'])){
            echo 'not found notificationAttempt';
            $error = true;
        }  

        if(!isset($response['paymentType']) OR empty($response['paymentType'])) {
            echo 'not found paymentType';
            $error = true;
        }   

        if(!isset($response['apiVersion']) OR empty($response['apiVersion'])) {
            echo 'not found apiVersion';
            $error = true;
        }   

        if(!isset($error)) {

            $sign = sprintf("%s|%s|%s|%s|%.2f|%d|%s|%d", $this -> get_option( 'paybylink_key' ), $response['transactionId'], $response['control'], $response['email'], $response['amountPaid'], $response['notificationAttempt'], $response['paymentType'], $response['apiVersion']);


                if(hash('sha256', $sign) != $response['signature']) {  
                    echo 'BAD SIGNATURE';
                    $error = true;
                } else {    
                    $order_id = explode("|", $response['control']);
                    $order = wc_get_order($order_id[1]);

                if ($order->get_status() !== 'completed' && $order->get_status() !== 'processing') {
                    $order->payment_complete($response['transactionId']);
                    $order->add_order_note( __( 'Płatności Paybylink na kwotę ' . $response['amountPaid'] . ' PLN została przyjęta.', 'paybylink_payment' ) );
                }

                header("HTTP/2.0 200 OK");
                echo 'OK';
                exit();

            }
        }
    }


    public function init_form_fields( )
    {
        $this -> form_fields = array(
             'enabled' => array(
                 'title' => __( 'Włączony / Wyłączony', 'paybylink_payment' ),
                'label' => __( 'Włącz metodę płatności', 'paybylink_payment' ),
                'type' => 'checkbox',
                'default' => 'no' 
            ),
            'title' => array(
                 'title' => __( 'Tytuł', 'paybylink_payment' ),
                'type' => 'text',
                'default' => __( '[Paybylink.pl] Płatność online oraz BLIK', 'woocommerce' ),
                'desc_tip' => __( 'Tytuł płatności widoczny dla użytkownika w momencie wybierania metody płatności', 'paybylink_payment' ) 
            ),
            'paybylink_id' => array(
                 'title' => __( 'Identyfikator Punktu Płatności', 'paybylink_payment' ),
                'type' => 'text',
                'default' => __( 'Identyfikator Punktu Płatności', 'woocommerce' ),
                'desc_tip' => __( 'Identyfikator Punktu Płatności znajdziesz w panelu swojego Punktu Płatności.', 'paybylink_payment' ) 
            ),
            'paybylink_key' => array(
                 'title' => __( 'Klucz Punktu Płatności', 'paybylink_payment' ),
                'type' => 'text',
                'default' => __( 'Klucz Punktu Płatności', 'woocommerce' ),
                'desc_tip' => __( 'Klucz Punktu Płatności znajdziesz w panelu swojego Punktu Płatności.', 'paybylink_payment' ) 
            )
        );
    }

     public function process_payment( $order_id )
    {
        global $woocommerce;
        $order = new WC_Order( $order_id );
      //  $woocommerce->cart->empty_cart();

        $shopId = $this -> get_option( 'paybylink_id' );
        
        $description = 'Płatność za zamówienie ' . $order_id;
        $this   ->  setControl( 'ORDER_ID|' . $order_id );
        $this   ->  setEmail( $order->billing_email );
        $this   ->  setNotifyUrl ( str_replace( 'http:', 'https:', add_query_arg( 'wc-api', 'paybylink_payment', home_url( '/' ) ) ) );
        $this   ->  setReturnUrlSuccess ( $this -> get_return_url( $order ) );
        $price  =   floatval($order->order_total);

        $data   = $this -> get_option( 'paybylink_key' ) . "|" . $shopId . "|" . sprintf("%.2f", $price);
        if ($this -> control != null && $this -> control != "") {
            $data .= "|" . $this -> control;
        }

        if ($description != null && $description != "") {
            $data .= "|" . $description;
        }

        if ($this -> email != null && $this -> email != "") {
            $data .= "|" . $this -> email;
        }

        if ($this -> notifyURL != null && $this -> notifyURL != "") {
            $data .= "|" . $this -> notifyURL;
        }

        if ($this -> returnUrlSuccess != null && $this -> returnUrlSuccess != "") {
            $data .= "|" . $this -> returnUrlSuccess;
        }

        $signature = hash('sha256', $data);
     
        $paybylinkData = array(
            'shopId'            => intval($shopId),
            'price'             => $price,
            'control'           => $this -> control,
            'description'       => $description,
            'email'             => $this -> email,
            'notifyURL'         => $this -> notifyURL,
            'returnUrlSuccess'  => $this -> returnUrlSuccess,
            'signature'         => $signature
        );

        $paybylinkData = json_encode($paybylinkData);   

        $response = wp_remote_post( 'https://secure.pbl.pl/api/v1/transfer/generate', array(
            'method'    => 'POST',
            'timeout'   => 90,
            'body'      => $paybylinkData,
            'headers'   => array(
                 "Content-Type" => "application/json;charset=UTF-8" 
            ),
            'sslverify' => false 
        ) );
        $response = json_decode( $response[ 'body' ] );
        $order->add_order_note( __( 'Nowa wpłata Paybylink', 'paybylink_payment' ) );

        return array(
          'result' => 'success',
            'redirect' => $response->url 
         );
    }

    
    public function admin_options()
    {
        ?>
    
        <h3><?php echo (! empty($this->method_title)) ? $this->method_title : __('Settings', 'woocommerce') ; ?></h3>
        <?php echo (! empty($this->method_description)) ? wpautop($this->method_description) : ''; ?>

        <div style="margin-bottom:40px;">
            <fieldset>
                <h3>Adres serwerowego potwierdzenia transakcji:</h3>
                <input id="woocommerce_paybylink" class="input-text regular-input " type="text" placeholder="" value="<?=  str_replace('http:', 'https:', add_query_arg('wc-api', 'paybylink_payment', home_url('/'))) ?>" style="" name="woocommerce_paybylink_connect_url" size="70" readonly/>
            </fieldset>
        </div>


    <table class="form-table">
        <h3>Panel Administracyjny</h3>
        <?php $this->generate_settings_html(); ?>
    </table>
        <?php
    }
}
